"""Graph test.

Code from https://github.com/RDFLib/rdflib/blob/91037207580838e41c07eb457bd65d7cc6d6ed85/test/test_graph.py

Copyright (c) 2002-2017, RDFLib Team
See CONTRIBUTORS and urn:github.com/RDFLib/rdflib
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above
copyright notice, this list of conditions and the following
disclaimer in the documentation and/or other materials provided
with the distribution.

  * Neither the name of Daniel Krech nor the names of its
contributors may be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

import unittest

from rdflib import Graph, URIRef


class GraphTestCase(unittest.TestCase):
    def setUp(self):
        self.graph = Graph(store="Oxigraph")
        self.michel = URIRef("urn:michel")
        self.tarek = URIRef("urn:tarek")
        self.bob = URIRef("urn:bob")
        self.likes = URIRef("urn:likes")
        self.hates = URIRef("urn:hates")
        self.pizza = URIRef("urn:pizza")
        self.cheese = URIRef("urn:cheese")

    def add_stuff(self):
        tarek = self.tarek
        michel = self.michel
        bob = self.bob
        likes = self.likes
        hates = self.hates
        pizza = self.pizza
        cheese = self.cheese

        self.graph.add((tarek, likes, pizza))
        self.graph.add((tarek, likes, cheese))
        self.graph.add((michel, likes, pizza))
        self.graph.add((michel, likes, cheese))
        self.graph.addN(
            [(bob, likes, cheese, self.graph), (bob, hates, pizza, self.graph), (bob, hates, michel, self.graph)]
        )

    def remove_stuff(self):
        tarek = self.tarek
        michel = self.michel
        bob = self.bob
        likes = self.likes
        hates = self.hates
        pizza = self.pizza
        cheese = self.cheese

        self.graph.remove((tarek, likes, pizza))
        self.graph.remove((tarek, likes, cheese))
        self.graph.remove((michel, likes, pizza))
        self.graph.remove((michel, likes, cheese))
        self.graph.remove((bob, likes, cheese))
        self.graph.remove((bob, hates, pizza))
        self.graph.remove((bob, hates, michel))  # gasp!

    def test_add(self):
        self.add_stuff()

    def test_remove(self):
        self.add_stuff()
        self.remove_stuff()

    def test_triples(self):
        tarek = self.tarek
        michel = self.michel
        bob = self.bob
        likes = self.likes
        hates = self.hates
        pizza = self.pizza
        cheese = self.cheese
        triples = self.graph.triples

        self.add_stuff()

        # unbound subjects
        self.assertEqual(len(list(triples((None, likes, pizza)))), 2)
        self.assertEqual(len(list(triples((None, hates, pizza)))), 1)
        self.assertEqual(len(list(triples((None, likes, cheese)))), 3)
        self.assertEqual(len(list(triples((None, hates, cheese)))), 0)

        # unbound objects
        self.assertEqual(len(list(triples((michel, likes, None)))), 2)
        self.assertEqual(len(list(triples((tarek, likes, None)))), 2)
        self.assertEqual(len(list(triples((bob, hates, None)))), 2)
        self.assertEqual(len(list(triples((bob, likes, None)))), 1)

        # unbound predicates
        self.assertEqual(len(list(triples((michel, None, cheese)))), 1)
        self.assertEqual(len(list(triples((tarek, None, cheese)))), 1)
        self.assertEqual(len(list(triples((bob, None, pizza)))), 1)
        self.assertEqual(len(list(triples((bob, None, michel)))), 1)

        # unbound subject, objects
        self.assertEqual(len(list(triples((None, hates, None)))), 2)
        self.assertEqual(len(list(triples((None, likes, None)))), 5)

        # unbound predicates, objects
        self.assertEqual(len(list(triples((michel, None, None)))), 2)
        self.assertEqual(len(list(triples((bob, None, None)))), 3)
        self.assertEqual(len(list(triples((tarek, None, None)))), 2)

        # unbound subjects, predicates
        self.assertEqual(len(list(triples((None, None, pizza)))), 3)
        self.assertEqual(len(list(triples((None, None, cheese)))), 3)
        self.assertEqual(len(list(triples((None, None, michel)))), 1)

        # all unbound
        self.assertEqual(len(list(triples((None, None, None)))), 7)
        self.remove_stuff()
        self.assertEqual(len(list(triples((None, None, None)))), 0)

    def test_connected(self):
        graph = self.graph
        self.add_stuff()
        self.assertEqual(True, graph.connected())

        jeroen = URIRef("urn:jeroen")
        unconnected = URIRef("urn:unconnected")

        graph.add((jeroen, self.likes, unconnected))

        self.assertEqual(False, graph.connected())

    def test_sub(self):
        g1 = self.graph
        g2 = Graph(store=g1.store)

        tarek = self.tarek
        bob = self.bob
        likes = self.likes
        pizza = self.pizza
        cheese = self.cheese

        g1.add((tarek, likes, pizza))
        g1.add((bob, likes, cheese))

        g2.add((bob, likes, cheese))

        g3 = g1 - g2

        self.assertEqual(len(g3), 1)
        self.assertEqual((tarek, likes, pizza) in g3, True)
        self.assertEqual((tarek, likes, cheese) in g3, False)

        self.assertEqual((bob, likes, cheese) in g3, False)

        g1 -= g2

        self.assertEqual(len(g1), 1)
        self.assertEqual((tarek, likes, pizza) in g1, True)
        self.assertEqual((tarek, likes, cheese) in g1, False)

        self.assertEqual((bob, likes, cheese) in g1, False)

    def test_graph_add(self):
        g1 = self.graph
        g2 = Graph(store=g1.store)

        tarek = self.tarek
        bob = self.bob
        likes = self.likes
        pizza = self.pizza
        cheese = self.cheese

        g1.add((tarek, likes, pizza))

        g2.add((bob, likes, cheese))

        g3 = g1 + g2

        self.assertEqual(len(g3), 2)
        self.assertEqual((tarek, likes, pizza) in g3, True)
        self.assertEqual((tarek, likes, cheese) in g3, False)

        self.assertEqual((bob, likes, cheese) in g3, True)

        g1 += g2

        self.assertEqual(len(g1), 2)
        self.assertEqual((tarek, likes, pizza) in g1, True)
        self.assertEqual((tarek, likes, cheese) in g1, False)

        self.assertEqual((bob, likes, cheese) in g1, True)

    def test_graph_intersection(self):
        g1 = self.graph
        g2 = Graph(store=g1.store)

        tarek = self.tarek
        michel = self.michel
        bob = self.bob
        likes = self.likes
        pizza = self.pizza
        cheese = self.cheese

        g1.add((tarek, likes, pizza))
        g1.add((michel, likes, cheese))

        g2.add((bob, likes, cheese))
        g2.add((michel, likes, cheese))

        g3 = g1 * g2

        self.assertEqual(len(g3), 1)
        self.assertEqual((tarek, likes, pizza) in g3, False)
        self.assertEqual((tarek, likes, cheese) in g3, False)

        self.assertEqual((bob, likes, cheese) in g3, False)
        self.assertEqual((michel, likes, cheese) in g3, True)

        g1 *= g2

        self.assertEqual(len(g1), 1)

        self.assertEqual((tarek, likes, pizza) in g1, False)
        self.assertEqual((tarek, likes, cheese) in g1, False)

        self.assertEqual((bob, likes, cheese) in g1, False)

        self.assertEqual((michel, likes, cheese) in g1, True)


if __name__ == "__main__":
    unittest.main()



"""Graph context test.

Code from https://github.com/RDFLib/rdflib/blob/91037207580838e41c07eb457bd65d7cc6d6ed85/test/test_graph_context.py

Copyright (c) 2002-2017, RDFLib Team
See CONTRIBUTORS and urn:github.com/RDFLib/rdflib
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above
copyright notice, this list of conditions and the following
disclaimer in the documentation and/or other materials provided
with the distribution.

  * Neither the name of Daniel Krech nor the names of its
contributors may be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

import unittest

from rdflib import BNode, ConjunctiveGraph, Graph, URIRef


class ContextTestCase(unittest.TestCase):
    def setUp(self):
        self.graph = ConjunctiveGraph(store="Oxigraph")
        self.michel = URIRef("urn:michel")
        self.tarek = URIRef("urn:tarek")
        self.bob = URIRef("urn:bob")
        self.likes = URIRef("urn:likes")
        self.hates = URIRef("urn:hates")
        self.pizza = URIRef("urn:pizza")
        self.cheese = URIRef("urn:cheese")

        self.c1 = URIRef("urn:context-1")
        self.c2 = URIRef("urn:context-2")

        # delete the graph for each test!
        self.graph.remove((None, None, None))

    def add_stuff(self):
        tarek = self.tarek
        michel = self.michel
        bob = self.bob
        likes = self.likes
        hates = self.hates
        pizza = self.pizza
        cheese = self.cheese
        c1 = self.c1
        graph = Graph(self.graph.store, c1)

        graph.add((tarek, likes, pizza))
        graph.add((tarek, likes, cheese))
        graph.add((michel, likes, pizza))
        graph.add((michel, likes, cheese))
        self.graph.addN([(bob, likes, cheese, graph), (bob, hates, pizza, graph), (bob, hates, michel, graph)])

    def remove_stuff(self):
        tarek = self.tarek
        michel = self.michel
        bob = self.bob
        likes = self.likes
        hates = self.hates
        pizza = self.pizza
        cheese = self.cheese
        c1 = self.c1
        graph = Graph(self.graph.store, c1)

        graph.remove((tarek, likes, pizza))
        graph.remove((tarek, likes, cheese))
        graph.remove((michel, likes, pizza))
        graph.remove((michel, likes, cheese))
        graph.remove((bob, likes, cheese))
        graph.remove((bob, hates, pizza))
        graph.remove((bob, hates, michel))  # gasp!

    def add_stuff_in_multiple_contexts(self):
        c1 = self.c1
        c2 = self.c2
        triple = (self.pizza, self.hates, self.tarek)  # revenge!

        # add to default context
        self.graph.add(triple)
        # add to context 1
        graph = Graph(self.graph.store, c1)
        graph.add(triple)
        # add to context 2
        graph = Graph(self.graph.store, c2)
        graph.add(triple)

    def test_conjunction(self):
        self.add_stuff_in_multiple_contexts()
        triple = (self.pizza, self.likes, self.pizza)
        # add to context 1
        graph = Graph(self.graph.store, self.c1)
        graph.add(triple)
        self.assertEqual(len(self.graph), len(graph))

    def test_add(self):
        self.add_stuff()

    def test_remove(self):
        self.add_stuff()
        self.remove_stuff()

    def test_len_in_one_context(self):
        c1 = self.c1
        # make sure context is empty

        self.graph.remove_context(self.graph.get_context(c1))
        graph = Graph(self.graph.store, c1)
        old_len = len(self.graph)

        for _ in range(0, 10):
            graph.add((BNode(), self.hates, self.hates))
        self.assertEqual(len(graph), old_len + 10)
        self.assertEqual(len(self.graph.get_context(c1)), old_len + 10)
        self.graph.remove_context(self.graph.get_context(c1))
        self.assertEqual(len(self.graph), old_len)
        self.assertEqual(len(graph), 0)

    def test_len_in_multiple_contexts(self):
        old_len = len(self.graph)
        self.add_stuff_in_multiple_contexts()

        # addStuffInMultipleContexts is adding the same triple to
        # three different contexts. So it's only + 1
        self.assertEqual(len(self.graph), old_len + 1)

        graph = Graph(self.graph.store, self.c1)
        self.assertEqual(len(graph), old_len + 1)

    def test_remove_in_multiple_contexts(self):
        c1 = self.c1
        c2 = self.c2
        triple = (self.pizza, self.hates, self.tarek)  # revenge!

        self.add_stuff_in_multiple_contexts()

        # triple should be still in store after removing it from c1 + c2
        self.assertTrue(triple in self.graph)
        graph = Graph(self.graph.store, c1)
        graph.remove(triple)
        self.assertTrue(triple in self.graph)
        graph = Graph(self.graph.store, c2)
        graph.remove(triple)
        self.assertTrue(triple in self.graph)
        self.graph.remove(triple)
        # now gone!
        self.assertTrue(triple not in self.graph)

        # add again and see if remove without context removes all triples!
        self.add_stuff_in_multiple_contexts()
        self.graph.remove(triple)
        self.assertTrue(triple not in self.graph)

    def test_contexts(self):
        triple = (self.pizza, self.hates, self.tarek)  # revenge!

        self.add_stuff_in_multiple_contexts()

        def cid(c):
            return c.identifier

        self.assertTrue(self.c1 in map(cid, self.graph.contexts()))
        self.assertTrue(self.c2 in map(cid, self.graph.contexts()))

        context_list = list(map(cid, list(self.graph.contexts(triple))))
        self.assertTrue(self.c1 in context_list, (self.c1, context_list))
        self.assertTrue(self.c2 in context_list, (self.c2, context_list))

    def test_remove_context(self):
        c1 = self.c1

        self.add_stuff_in_multiple_contexts()
        self.assertEqual(len(Graph(self.graph.store, c1)), 1)
        self.assertEqual(len(self.graph.get_context(c1)), 1)

        self.graph.remove_context(self.graph.get_context(c1))
        self.assertTrue(self.c1 not in self.graph.contexts())

    def test_remove_any(self):
        self.add_stuff_in_multiple_contexts()
        self.graph.remove((None, None, None))
        self.assertEqual(len(self.graph), 0)

    def test_triples(self):
        tarek = self.tarek
        michel = self.michel
        bob = self.bob
        likes = self.likes
        hates = self.hates
        pizza = self.pizza
        cheese = self.cheese
        c1 = self.c1
        triples = self.graph.triples
        graph = self.graph
        c1graph = Graph(self.graph.store, c1)
        c1triples = c1graph.triples

        self.add_stuff()

        # unbound subjects with context
        self.assertEqual(len(list(c1triples((None, likes, pizza)))), 2)
        self.assertEqual(len(list(c1triples((None, hates, pizza)))), 1)
        self.assertEqual(len(list(c1triples((None, likes, cheese)))), 3)
        self.assertEqual(len(list(c1triples((None, hates, cheese)))), 0)

        # unbound subjects without context, same results!
        self.assertEqual(len(list(triples((None, likes, pizza)))), 2)
        self.assertEqual(len(list(triples((None, hates, pizza)))), 1)
        self.assertEqual(len(list(triples((None, likes, cheese)))), 3)
        self.assertEqual(len(list(triples((None, hates, cheese)))), 0)

        # unbound objects with context
        self.assertEqual(len(list(c1triples((michel, likes, None)))), 2)
        self.assertEqual(len(list(c1triples((tarek, likes, None)))), 2)
        self.assertEqual(len(list(c1triples((bob, hates, None)))), 2)
        self.assertEqual(len(list(c1triples((bob, likes, None)))), 1)

        # unbound objects without context, same results!
        self.assertEqual(len(list(triples((michel, likes, None)))), 2)
        self.assertEqual(len(list(triples((tarek, likes, None)))), 2)
        self.assertEqual(len(list(triples((bob, hates, None)))), 2)
        self.assertEqual(len(list(triples((bob, likes, None)))), 1)

        # unbound predicates with context
        self.assertEqual(len(list(c1triples((michel, None, cheese)))), 1)
        self.assertEqual(len(list(c1triples((tarek, None, cheese)))), 1)
        self.assertEqual(len(list(c1triples((bob, None, pizza)))), 1)
        self.assertEqual(len(list(c1triples((bob, None, michel)))), 1)

        # unbound predicates without context, same results!
        self.assertEqual(len(list(triples((michel, None, cheese)))), 1)
        self.assertEqual(len(list(triples((tarek, None, cheese)))), 1)
        self.assertEqual(len(list(triples((bob, None, pizza)))), 1)
        self.assertEqual(len(list(triples((bob, None, michel)))), 1)

        # unbound subject, objects with context
        self.assertEqual(len(list(c1triples((None, hates, None)))), 2)
        self.assertEqual(len(list(c1triples((None, likes, None)))), 5)

        # unbound subject, objects without context, same results!
        self.assertEqual(len(list(triples((None, hates, None)))), 2)
        self.assertEqual(len(list(triples((None, likes, None)))), 5)

        # unbound predicates, objects with context
        self.assertEqual(len(list(c1triples((michel, None, None)))), 2)
        self.assertEqual(len(list(c1triples((bob, None, None)))), 3)
        self.assertEqual(len(list(c1triples((tarek, None, None)))), 2)

        # unbound predicates, objects without context, same results!
        self.assertEqual(len(list(triples((michel, None, None)))), 2)
        self.assertEqual(len(list(triples((bob, None, None)))), 3)
        self.assertEqual(len(list(triples((tarek, None, None)))), 2)

        # unbound subjects, predicates with context
        self.assertEqual(len(list(c1triples((None, None, pizza)))), 3)
        self.assertEqual(len(list(c1triples((None, None, cheese)))), 3)
        self.assertEqual(len(list(c1triples((None, None, michel)))), 1)

        # unbound subjects, predicates without context, same results!
        self.assertEqual(len(list(triples((None, None, pizza)))), 3)
        self.assertEqual(len(list(triples((None, None, cheese)))), 3)
        self.assertEqual(len(list(triples((None, None, michel)))), 1)

        # all unbound with context
        self.assertEqual(len(list(c1triples((None, None, None)))), 7)
        # all unbound without context, same result!
        self.assertEqual(len(list(triples((None, None, None)))), 7)

        for c in [graph, self.graph.get_context(c1)]:
            # unbound subjects
            self.assertEqual(set(c.subjects(likes, pizza)), {michel, tarek})
            self.assertEqual(set(c.subjects(hates, pizza)), {bob})
            self.assertEqual(set(c.subjects(likes, cheese)), {tarek, bob, michel})
            self.assertEqual(set(c.subjects(hates, cheese)), set())

            # unbound objects
            self.assertEqual(set(c.objects(michel, likes)), {cheese, pizza})
            self.assertEqual(set(c.objects(tarek, likes)), {cheese, pizza})
            self.assertEqual(set(c.objects(bob, hates)), {michel, pizza})
            self.assertEqual(set(c.objects(bob, likes)), {cheese})

            # unbound predicates
            self.assertEqual(set(c.predicates(michel, cheese)), {likes})
            self.assertEqual(set(c.predicates(tarek, cheese)), {likes})
            self.assertEqual(set(c.predicates(bob, pizza)), {hates})
            self.assertEqual(set(c.predicates(bob, michel)), {hates})

            self.assertEqual(set(c.subject_objects(hates)), {(bob, pizza), (bob, michel)})
            self.assertEqual(
                set(c.subject_objects(likes)),
                {(tarek, cheese), (michel, cheese), (michel, pizza), (bob, cheese), (tarek, pizza)},
            )

            self.assertEqual(set(c.predicate_objects(michel)), {(likes, cheese), (likes, pizza)})
            self.assertEqual(set(c.predicate_objects(bob)), {(likes, cheese), (hates, pizza), (hates, michel)})
            self.assertEqual(set(c.predicate_objects(tarek)), {(likes, cheese), (likes, pizza)})

            self.assertEqual(set(c.subject_predicates(pizza)), {(bob, hates), (tarek, likes), (michel, likes)})
            self.assertEqual(set(c.subject_predicates(cheese)), {(bob, likes), (tarek, likes), (michel, likes)})
            self.assertEqual(set(c.subject_predicates(michel)), {(bob, hates)})

            self.assertEqual(
                set(c),
                {
                    (bob, hates, michel),
                    (bob, likes, cheese),
                    (tarek, likes, pizza),
                    (michel, likes, pizza),
                    (michel, likes, cheese),
                    (bob, hates, pizza),
                    (tarek, likes, cheese),
                },
            )

        # remove stuff and make sure the graph is empty again
        self.remove_stuff()
        self.assertEqual(len(list(c1triples((None, None, None)))), 0)
        self.assertEqual(len(list(triples((None, None, None)))), 0)


if __name__ == "__main__":
    unittest.main()



import unittest
from pathlib import Path

from pyoxigraph import Store
from rdflib import RDF, XSD, BNode, ConjunctiveGraph, Graph, Literal, Namespace

from oxrdflib import OxigraphStore

EX = Namespace("http://example.com/")


class StoreTestCase(unittest.TestCase):
    def test_store_without_open(self):
        g = ConjunctiveGraph("Oxigraph")
        self._fill_graph(g)
        self._test_graph(g)
        self.assertEqual(len(list(iter(g))), 4)

    def test_store_with_open(self):
        g = ConjunctiveGraph("Oxigraph")
        g.open("test_store")
        self._fill_graph(g)
        g.close()
        del g
        self.assertTrue(Path("test_store").exists())

        g = ConjunctiveGraph("Oxigraph")
        g.open("test_store")
        self._test_graph(g)
        g.close()
        g.destroy("test_store")
        self.assertFalse(Path("test_store").exists())

    def test_store_with_late_open(self):
        g = ConjunctiveGraph("Oxigraph")
        g.add((EX.foo, RDF.type, EX.Entity))
        with self.assertRaises(Exception) as _:
            g.open("test_store")

    def test_store_with_shared_backend(self):
        store = Store()
        self._fill_graph(Graph(store=OxigraphStore(store=store), identifier="http://example.com"))
        self._test_graph(Graph(store=OxigraphStore(store=store), identifier="http://example.com"))

    @staticmethod
    def _fill_graph(g: Graph):
        g.add((EX.foo, RDF.type, EX.Entity))
        g.add((EX.foo, EX.prop, BNode("123")))
        g.add((EX.foo, EX.prop1, Literal("foo")))
        g.add((EX.foo, EX.prop1, Literal("foo", lang="en")))
        g.add((EX.foo, EX.prop1, Literal("1", datatype=XSD.integer)))
        g.remove((EX.foo, EX.prop, BNode("123")))

    def _test_graph(self, g: Graph):
        self.assertIn((EX.foo, RDF.type, EX.Entity), g)
        self.assertNotIn((EX.foo, EX.prop, EX.bar), g)
        self.assertEqual(len(g), 4)


if __name__ == "__main__":
    unittest.main()



import json
import unittest

from rdflib import RDF, ConjunctiveGraph, Graph, Namespace

EX = Namespace("http://example.com/")


class SparqlTestCase(unittest.TestCase):
    def test_ask_query(self):
        g = ConjunctiveGraph("Oxigraph")
        g.add((EX.foo, RDF.type, EX.Entity))
        g.bind("ex", EX)

        # basic
        result = g.query("ASK { ?s ?p ?o }")
        self.assertTrue(result)
        self.assertIsInstance(result.serialize(), bytes)

        # with not initialized prefix
        self.assertTrue(g.query("ASK { ex:foo rdf:type ex2:Entity }", initNs={"ex2": EX}))

        # with init entities
        self.assertFalse(g.query("ASK { ?s ?p ?o }", initBindings={"o": EX.NotExists}))

        # in specific graph
        g = ConjunctiveGraph("Oxigraph")
        g1 = Graph(store=g.store, identifier=EX.g1)
        g1.add((EX.foo, RDF.type, EX.Entity))
        self.assertTrue(g1.query("ASK { ?s ?p ?o }"))

    def test_select_query_graph(self):
        g = Graph("Oxigraph")
        g.add((EX.foo, RDF.type, EX.Entity))
        result = g.query("SELECT ?s WHERE { ?s ?p ?o }")
        self.assertEqual(len(result), 1)
        self.assertEqual(
            json.loads(result.serialize(format="json").decode("utf-8")),
            {
                "results": {"bindings": [{"s": {"type": "uri", "value": "http://example.com/foo"}}]},
                "head": {"vars": ["s"]},
            },
        )

    def test_select_query_conjunctive(self):
        g = ConjunctiveGraph("Oxigraph")
        g.add((EX.foo, RDF.type, EX.Entity))
        result = g.query("SELECT ?s WHERE { ?s ?p ?o }")
        self.assertEqual(len(result), 1)
        self.assertEqual(
            json.loads(result.serialize(format="json").decode("utf-8")),
            {
                "results": {"bindings": [{"s": {"type": "uri", "value": "http://example.com/foo"}}]},
                "head": {"vars": ["s"]},
            },
        )

    def test_construct_query(self):
        g = ConjunctiveGraph("Oxigraph")
        g.add((EX.foo, RDF.type, EX.Entity))
        result = g.query("CONSTRUCT WHERE { ?s ?p ?o }")
        self.assertEqual(len(result), 1)
        self.assertEqual(
            result.serialize(format="ntriples").strip(),
            b"<http://example.com/foo> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://example.com/Entity> .",
        )


if __name__ == "__main__":
    unittest.main()



"""Dataset test.

Code from https://github.com/RDFLib/rdflib/blob/91037207580838e41c07eb457bd65d7cc6d6ed85/test/test_dataset.py

Copyright (c) 2002-2017, RDFLib Team
See CONTRIBUTORS and urn:github.com/RDFLib/rdflib
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above
copyright notice, this list of conditions and the following
disclaimer in the documentation and/or other materials provided
with the distribution.

  * Neither the name of Daniel Krech nor the names of its
contributors may be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

import unittest

from rdflib import Dataset, URIRef
from rdflib.graph import DATASET_DEFAULT_GRAPH_ID


class DatasetTestCase(unittest.TestCase):
    def setUp(self):
        self.graph = Dataset(store="Oxigraph")
        self.michel = URIRef("urn:michel")
        self.tarek = URIRef("urn:tarek")
        self.bob = URIRef("urn:bob")
        self.likes = URIRef("urn:likes")
        self.hates = URIRef("urn:hates")
        self.pizza = URIRef("urn:pizza")
        self.cheese = URIRef("urn:cheese")

        # Use regular URIs because SPARQL endpoints like Fuseki alter short names
        self.c1 = URIRef("urn:context-1")
        self.c2 = URIRef("urn:context-2")

        # delete the graph for each test!
        self.graph.remove((None, None, None))
        for c in self.graph.contexts():
            c.remove((None, None, None))
            self.assertEqual(len(c), 0)
            self.graph.remove_graph(c)

    def tearDown(self):
        self.graph.close()

    def test_graph_aware(self):
        if not self.graph.store.graph_aware:
            return

        g = self.graph
        g1 = g.graph(self.c1)

        # added graph exists
        self.assertEqual(
            {x.identifier for x in self.graph.contexts()},
            {self.c1, DATASET_DEFAULT_GRAPH_ID},
        )

        # added graph is empty
        self.assertEqual(len(g1), 0)

        g1.add((self.tarek, self.likes, self.pizza))

        # added graph still exists
        self.assertEqual(
            {x.identifier for x in self.graph.contexts()},
            {self.c1, DATASET_DEFAULT_GRAPH_ID},
        )

        # added graph contains one triple
        self.assertEqual(len(g1), 1)

        g1.remove((self.tarek, self.likes, self.pizza))

        # added graph is empty
        self.assertEqual(len(g1), 0)

        # graph still exists, although empty
        self.assertEqual(
            {x.identifier for x in self.graph.contexts()},
            {self.c1, DATASET_DEFAULT_GRAPH_ID},
        )

        g.remove_graph(self.c1)

        # graph is gone
        self.assertEqual(
            {x.identifier for x in self.graph.contexts()},
            {DATASET_DEFAULT_GRAPH_ID},
        )

    def test_default_graph(self):
        self.graph.add((self.tarek, self.likes, self.pizza))
        self.assertEqual(len(self.graph), 1)
        # only default exists
        self.assertEqual(
            {x.identifier for x in self.graph.contexts()},
            {DATASET_DEFAULT_GRAPH_ID},
        )

        # removing default graph removes triples but not actual graph
        self.graph.remove_graph(DATASET_DEFAULT_GRAPH_ID)

        self.assertEqual(len(self.graph), 0)
        # default still exists
        self.assertEqual(
            {x.identifier for x in self.graph.contexts()},
            {DATASET_DEFAULT_GRAPH_ID},
        )

    def test_not_union(self):
        g1 = self.graph.graph(self.c1)
        g1.add((self.tarek, self.likes, self.pizza))

        self.assertEqual(list(self.graph.objects(self.tarek, None)), [])
        self.assertEqual(list(g1.objects(self.tarek, None)), [self.pizza])


if __name__ == "__main__":
    unittest.main()
